/*************************************************************************
*
* ADOBE SYSTEMS INCORPORATED
* Copyright 2001 Adobe Systems Incorporated 
* All Rights Reserved

* NOTICE: Adobe permits you to use, modify, and distribute 
* this file in accordance with the terms of the Adobe license 
* agreement accompanying it. If you have received this file 
* from a source other than Adobe, then your use, modification, 
* or distribution of it requires the prior written permission 
* of Adobe.
*
**************************************************************************/

#ifndef _JSA_H_
#define _JSA_H_

/**
The JSA interface permits developers to write binary code extensions for the Javascript SDK.
All modules must reside in the Modules/Extend Scripts/Common folder, and all functions are 
callable by all Javascript extension modules. The functions are accessible as parts of a Javscript object
that has the same name as the DLL or SharedLib. If, for example, the DLL is called "SomeExt.DLL",
and it contains a function "foo", the function is callable as "SomeExt.foo()".
<p>
The parameters and return values of binary functions are freely definable, as Javascript supplies
and parameters given as is. There is one exception, however. If an object is supplied, Javascript
converts the objects to a scalar value by first valling its valueOf() method, and, if that method
again returned an object (which is the default for most objects), its toString() method. Also,
it is not possible for binary functions to return object references. A function implementation
is of the type JSANativeMethod, which is a void function receiving an argc/argv parameter combination
and a pointer to store any return result. Pointers to values are opaque void pointers, and there
are a number of callback macros available to access these macros.
<p>
When writing a module, it is important to implement the macro JSA_INIT once that defines some data
structures and performs the necessary initialization of the module.
*/

#ifdef __cplusplus			
extern "C" {				
#endif						

/// All data elements are opaque void pointers that are casted internally.

typedef void *JSValue;

/// The supplied data is one of the following scalar types, returned by JSAGetValueType().

enum JSAValueType {
	/// Undefined or empty value.
	JSA_UNDEFINED,
	/// A boolean value,either 0 or 1.
	JSA_BOOL,
	/// A 32 bit signed integer quantity.
	JSA_INTEGER,
	/// An 8 byte double precision floating point value.
	JSA_DOUBLE,
	/// A null-terminated ASCII string.
	JSA_STRING
};

// ---------- Function Definitions ------------

typedef struct _JSAEnv JSAEnv;

typedef void (*JSANativeMethod)(int argc, JSValue *argv, JSValue returnValue);

typedef int    (*JSAValueTypeFct)(JSAEnv *, JSValue);
typedef long   (*JSAValueToIntFct)(JSAEnv *, JSValue);
typedef void   (*JSAIntToValueFct)(JSAEnv *, long, JSValue);
typedef int    (*JSAValueToBoolFct)(JSAEnv *, JSValue);
typedef void   (*JSABoolToValueFct)(JSAEnv *, int, JSValue);
typedef char*  (*JSAValueToStringFct)(JSAEnv *, JSValue);
typedef void   (*JSAStringToValueFct)(JSAEnv *, char *, JSValue);
typedef double (*JSAValueToDoubleFct)(JSAEnv *, JSValue);
typedef void   (*JSADoubleToValueFct)(JSAEnv *, double, JSValue);
typedef void   (*JSARegisterFunctionFct)(JSAEnv *, char *name, JSANativeMethod);
typedef void   (*JSASetErrorFct)(JSAEnv *, char*);
typedef void   (*JSAUndefinedToValueFct)(JSAEnv *, JSValue);
typedef void   (*JSAEvalFct)(JSAEnv*, char* text, JSValue retVal, long timeout);
typedef void   (*JSAPollFct)(JSAEnv*);	// 50.003

#ifdef WIN32
#define JSAEXPORT __declspec(dllexport)
#else
#define JSAEXPORT
#endif

// ---------- Environment ---------------------

typedef struct _JSAEnv
{
	long					jsaStructSize;
	long					jsaVersion;		// version number = SDK version number * 1000 (f.ex. 50003 for 50.003)
	void *					jsaInst;
	void *					externalRef;
	JSAValueTypeFct			vtypFct;
	JSAValueToIntFct		vtoiFct;
	JSAIntToValueFct		itovFct;
	JSAValueToBoolFct		vtobFct;
	JSABoolToValueFct		btovFct;
	JSAValueToStringFct		vtosFct;
	JSAStringToValueFct		stovFct;
	JSAValueToDoubleFct		vtodFct;
	JSADoubleToValueFct		dtovFct;
	JSARegisterFunctionFct	regFct;
	JSASetErrorFct			errFct;
	JSAUndefinedToValueFct	utovFct;
	JSAEvalFct				evalFct;
	JSAPollFct				pollFct;		// added in version 50.003
} JSAEnv;
	
// ---------- Interface -------------------------

/**
The JSAMain function is called from within the JSA_INIT macro. It should be used
to register all functions callable by the Javascript SDK. It is called when the
module has been loaded, which the SDK does on demand before calling the first
function inside the module.
*/
extern	void JSAEXPORT JSAMain(void);

/**
The JSAExit function is called when the extension module is about to be unloaded.
*/
extern	void JSAEXPORT JSAExit(void);
/**
Retrieve the type of a value. It is one of the JSAValueType enums.
@param	v			the JSAValue to check
@return				the type of the value
*/
#define JSAGetValueType(v)				JSAEnvObj->vtypFct(JSAEnvObj,v)

/**
Retrieve the value as a signed 32 bit integer quantity. Automatic conversion
to the value is applied according to Javascript rules if necessary.
@param	v			the JSAValue to convert
@return				the integer value
*/
#define JSAValueToInt(v)				JSAEnvObj->vtoiFct(JSAEnvObj,v)

/**
Store an integer value into the given JSAValue.
@param	v			the JSAValue to receive the integer value
@param	i			the integer value to store
*/
#define JSAIntToValue(v,i)				JSAEnvObj->itovFct(JSAEnvObj,i,v)

/**
Retrieve the value as a boolean quantity (zero or nonzero). Automatic conversion
to the value is applied according to Javascript rules if necessary.
@param	v			the JSAValue to convert
@return				the boolean value
*/
#define JSAValueToBool(v)				JSAEnvObj->vtobFct(JSAEnvObj,v)

/**
Store an boolean value into the given JSAValue.
@param	v			the JSAValue to receive the boolean value (zero or nonzero)
@param	b			the boolean value (zero or nonzero) to store
*/
#define JSABoolToValue(v,b)				JSAEnvObj->btovFct(JSAEnvObj,b,v)

/**
Retrieve the value as a zero terminated ASCII string. Automatic conversion
to the value is applied according to Javascript rules if necessary.
@param	v			the JSAValue to convert
@return				the ASCII string
*/
#define JSAValueToString(v)				JSAEnvObj->vtosFct(JSAEnvObj,v)

/**
Store a zero terminated ASCII string value into the given JSAValue.
@param	v			the JSAValue to receive the string
@param	s			the string to store
*/
#define JSAStringToValue(v,s)			JSAEnvObj->stovFct(JSAEnvObj,s,v)

/**
Retrieve the value as an eight byte floating point (double) value. Automatic conversion
to the value is applied according to Javascript rules if necessary.
@param	v			the JSAValue to convert
@return				the floating point value
*/
#define JSAValueToDouble(v)				JSAEnvObj->vtodFct(JSAEnvObj,v)

/**
Store an eight byte floating point (double) value into the given JSAValue.
@param	v			the JSAValue to receive the floating point value
@param	d			the floating point value to store
*/
#define JSADoubleToValue(v,d)			JSAEnvObj->dtovFct(JSAEnvObj,d,v)

/**
Set the given value to undefined.
@param	v			the JSAValue to be set to undefined.
*/
#define JSAUndefinedToValue(v)			JSAEnvObj->utovFct(JSAEnvObj,v)

/**
Register a function by name within the Javascript extension. Only registered functions can
be called from within the Javascirpt extension modules. The function must be a JSANativeMethod.
The location to register functions is within the JSAMain() function.
@param	n			the function name. The name must follow Javascript naimg conventions.
@param	f			the function itself (a JSANativeMethod)
*/
#define JSARegisterFunction(n,f)		JSAEnvObj->regFct(JSAEnvObj,n,f)

/**
Set a runtime error.
@param	text		the error message as a zero terminated ASCII string
*/
#define JSASetError(text)				JSAEnvObj->errFct(JSAEnvObj,text)

/**
Evaluate a Javascript scriptlet. The scriptlet is executed within the current runtime scope
of Javascript. An optional JSAValue may be supplied to receive the return value. The given
timeout is the timeout in milliseconds to wait for the engine. If the timeout elapses before
this call returns, a runtime error is generated. Supplying 0 or a negative value means that
no timeout error will be generated. Keep in mind that calling this function may result in 
undesired side effects like recursion!
@param	text		the scriptlet code to run
@param	retval		the optional return value (may be NULL for no return value)
@param	timeout		the timeout in milliseconds
*/
#define	JSAEval(text,retval,timeout)	JSAEnvObj->evalFct(JSAEnvObj,text,retval,timeout)

/**
Poll the Javascript engine. This callback checks the engine for delayed scriptlets (scriptlets
defined with the startTimer() function) and executes them when they are due. Keep in mind that
calling this function may result in undesired side effects like recursion!
*/
#define	JSAPoll()						JSAEnvObj->pollFct(JSAEnvObj)

/**
The structure supplied to allow for drawing inside a module.
The Draw.getDrawInfo() method of the Javascript SDK returns a long value that is actually a
pointer to a JSADrawInfo structure. The pointer may be retrieved by JSAValueToInt() and casted
to a JSADrawInfo structure. The contents of the structure can be used to implement drawing
in native code.
*/
typedef struct _JSADrawInfo
{
	/// The device context. This is a DC handle on Windows or a GrafPtr pointer on the Mac.
	long		context;
	/// The top left corner of the current drawing area.
	long		left, top;
	/// The bottom right corner of the current drawing area.
	long		right, bottom;
} JSADrawInfo;

// ---------- Implementation ----------------

extern	JSAEnv	*JSAEnvObj;
extern	void JSAEXPORT JSAEntry(JSAEnv *env);

#define JSA_INIT			\
							\
JSAEnv	*JSAEnvObj;			\
							\
void main()					\
{							\
}							\
							\
void JSAEXPORT JSAEntry(JSAEnv *env)	\
{							\
	JSAEnvObj = env;		\
	JSAMain();				\
}							\

#ifdef __cplusplus
}
#endif

#endif
